//
//      black-hole-mta - email goes in but nothing comes out
//      Copyright (C) 2007, 2011 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License as
//      published by the Free Software Foundation; either version 3 of
//      the License, or (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef LIB_MAIL_TRANSFER_AGENT_H
#define LIB_MAIL_TRANSFER_AGENT_H

#include <boost/shared_ptr.hpp>
#include <list>
#include <map>
#include <string>

/**
  * The mail_transfer_agent abstract base class is used to represent the
  * API of a generic mail transfer agent.
  */
class mail_transfer_agent
{
public:
    /**
      * The pointer type shall always be used to point to
      * mail_transfer_agent instances.
      */
    typedef boost::shared_ptr<mail_transfer_agent> pointer;

    /**
      * The destructor.
      */
    virtual ~mail_transfer_agent();

    /**
      * The create class method is used to manufacture new mail transfer
      * agent instances.
      *
      * @param name
      *     The name of the type of mail transfer agent to create.  If
      *     no name is given, a black hole is created.
      */
    static pointer create(const char *name = 0);

    /**
      * The mailq method is used to implement the command line interface
      * of the mailq(1) command.
      *
      * @param argc
      *     The number of commend line arguments.
      * @param argv
      *     The value of each command line argument.
      */
    void mailq(int argc, char **argv);

    /**
      * The newaliases method is used to implement the command line
      * interface of the newaliases(1) command.
      *
      * @param argc
      *     The number of commend line arguments.
      * @param argv
      *     The value of each command line argument.
      */
    void newaliases(int argc, char **argv);

    /**
      * The sendmail method is used to implement the command line
      * interface of the sendmail(1) command.
      *
      * @param argc
      *     The number of commend line arguments.
      * @param argv
      *     The value of each command line argument.
      */
    void sendmail(int argc, char **argv);

    /**
      * The daemon method is used to implement the command line
      * interface of the smtpd(1) command.
      *
      * @param argc
      *     The number of commend line arguments.
      * @param argv
      *     The value of each command line argument.
      */
    void daemon(int argc, char **argv);

protected:
    /**
      * The mailq method is called by the command line interface to list
      * each entry of the mail queue.
      */
    virtual void mailq() = 0;

    /**
      * The newaliases method is called by the command line interface to
      * rebuild the email address alias database.
      */
    virtual void newaliases() = 0;

    /**
      * The send method is used to send email to the given address.
      *
      * @param address
      *     The email address to deliver the email to.
      * @param filename
      *     The file containing the email to be sent.
      */
    virtual void send(const std::string &email_address,
        const std::string &filename) = 0;

    typedef std::list<std::string> string_list_t;

    /**
      * The verify method is called by the sendmail command line
      * processing to implement the -bv action.
      *
      * @param names
      *     The names to be verified.
      */
    virtual void verify(const string_list_t &names) = 0;

    /**
      * The purge_host_status method is called by the sendmail command
      * line processing to implement the -bH action.
      */
    virtual void purge_host_status() = 0;

    /**
      * The print_host_status method is called by the sendmail command
      * line processing to implement the -bh action.
      */
    virtual void print_host_status() = 0;

    /**
      * The default constructor.  Only derived classes may call this;
      * there must always be a derived class.
      */
    mail_transfer_agent();

    typedef void (mail_transfer_agent::*be_method_pointer)(
        const string_list_t &names);

private:
    /**
      * The set method is used to set the named value in the options
      * database.
      *
      * @param name
      *     The name of the option.
      * @param value
      *     The value of the named option.
      */
    void set(const std::string &name, const std::string &value);

    /**
      * The set method is used to set the named value in the options
      * database.
      *
      * @param name
      *     The name of the option.
      * @param value
      *     The value of the named option.
      */
    void set(const std::string &name, bool value);

    /**
      * The set method is used to set the named value in the options
      * database, unless it is set already.
      *
      * @param name
      *     The name of the option.
      * @param value
      *     The value of the named option.
      */
    void set_unless(const std::string &name, const std::string &value);

    /**
      * The get method is used to obtain the value of the named option.
      *
      * @param name
      *     The name of the option.
      * @param dflt
      *     The default value of the option if it is not already present
      *     in the options database.
      * @returns
      *     The value of the named option.
      */
    std::string get(const std::string &name, const std::string &dflt = "");

    /**
      * The get_bool method is used to obtain the value of the named
      * boolean option.
      *
      * @param name
      *     The name of the option.
      * @param dflt
      *     The default value of the option if it is not already present
      *     in the options database.
      * @returns
      *     The value of the named option.
      */
    bool get_bool(const std::string &name, bool dflt = false);

private:
    /**
      * The parse_and_set method is used by the comamnd line parsing to
      * grok options settings.  It inderstands name=value and also the
      * legacy Nvalue (name is for letter) form.
      *
      * @param string
      *     The command line option value.
      */
    void parse_and_set(const char *string);

    /**
      * The set_configuration_file is called by the command line
      * processing to set the path name of the configuration file.
      *
      * @param path
      *     The path name of the configuration file.
      */
    void set_configuration_file(const char *path);

    /**
      * The configuration_file instance variable is used to remember the
      * path name of the configuration file.
      */
    std::string configuration_file;

    typedef std::map<std::string, std::string> db_t;

    /**
      * The db instance variable is used to remeber the values of
      * options, by name.
      */
    db_t db;

    /**
      * The mailq_glue method is called by the sendmail command line
      * interface to list each entry of the mail queue.  This is
      * a convenience method to support the sendmail command line
      * processing.
      *
      * @param names
      *     The names which appear on the command line (should be empty).
      */
    void mailq_glue(const string_list_t &names);

    /**
      * The newaliases_glue method is called by the sendmail command
      * line interface to rebuild the aliases database.  This is
      * a convenience method to support the sendmail command line
      * processing.
      *
      * @param names
      *     The names which appear on the command line (should be empty).
      */
    void newaliases_glue(const string_list_t &names);

    /**
      * The send_glue method is called by the sendmail command line
      * interface to send email to the given recipients.  It calls the
      * cirtual send(const std::string &address, const std::string
      * &filename) method to perform the actual delivery.
      *
      * @param names
      *     The names which appear on the command line.
      */
    void send_glue(const string_list_t &names);

    /**
      * The server method is used to read SMTP (rfc821) commands from
      * the standard input and write the responses to standatd output.
      * This could be manual or scripted testing, or it could be an
      * appropriately manipulated socket from the daemon() code.
      */
    void server();

    /**
      * The server_glue method is called by the sendmail command line
      * interface to server functionality.  It calls the actual server()
      * method to perform the actual functionality.
      *
      * @param names
      *     The names which appear on the command line (should be empty).
      */
    void server_glue(const string_list_t &names);

    /**
      * The daemon method is used to wait for port 25 connections, and
      * spawn a server for each one.
      */
    void daemon();

    /**
      * The daemon_glue method is called by the sendmail command line
      * interface to daemon functionality.  It calls the actual daemon()
      * method to perform the actual functionality.
      *
      * @param names
      *     The names which appear on the command line (should be empty).
      */
    void daemon_glue(const string_list_t &names);

    /**
      * The daemon_nofork_glue method is called by the sendmail command line
      * interface to daemon functionality.  It calls the daemon_glue()
      * method after setting "debug" to true.
      *
      * @param names
      *     The names which appear on the command line.
      */
    void daemon_nofork_glue(const string_list_t &names);

    /**
      * The no_names_on_command_line method is called by the command
      * line processing to issue a fatal error when names appear on the
      * command line when should be none.
      */
    void no_names_on_command_line();

    /**
      * The nop method is a placeholder for "be" options which are
      * ignored.  The sendmail_lookup method returns a pointer to it
      * when this happens.
      *
      * @param names
      *     The names which appear on the command line (should be empty).
      */
    void nop(const string_list_t &names);

    /**
      * The sendmail_lookup method is used to map a name to a method
      * pointer for use in the sendmail command line processing.
      *
      * @param name
      *     The name of the method.  For historical reasons, they tend
      *     to be cryptic single letters.
      * @returns
      *     pointer to method.
      */
    be_method_pointer sendmail_lookup(const char *name);

    /**
      * The usage_with_names method is used to print a fatal usage error
      * message, including names on the example command line.
      */
    void usage_with_names();

    /**
      * The usage_with_names method is used to print a fatal usage error
      * message. There will be no names on the example command line.
      */
    void usage_without_names();

    /**
      * The read_configuration_file method is used to read the
      * configuration file, if it has not been read already.
      */
    void read_configuration_file();

    /**
      * The have_read_configuration_file instance variable is used to
      * remember whether or not the configuration file has been read.
      */
    bool have_read_configuration_file;

    void purge_host_status_glue(const string_list_t &names);

    void print_host_status_glue(const string_list_t &names);

    /**
      * The copy constructor.  Do not use.
      */
    mail_transfer_agent(const mail_transfer_agent &);

    /**
      * The assignment operator.  Do not use.
      */
    mail_transfer_agent &operator=(const mail_transfer_agent &);
};

// vim: set ts=8 sw=4 et :
#endif // LIB_MAIL_TRANSFER_AGENT_H
