//
// black-hole-mta - email goes in but nothing comes out
// Copyright (C) 2007, 2009, 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/config.h>
#include <cerrno>
#include <csignal>
#include <cstdio>
#include <cstring>
#include <netdb.h>
#include <netinet/in.h>
#include <syslog.h>
#include <libexplain/accept.h>
#include <libexplain/bind.h>
#include <libexplain/dup2.h>
#include <libexplain/fork.h>
#include <libexplain/listen.h>
#include <libexplain/output.h>
#include <libexplain/program_name.h>
#include <libexplain/socket.h>
#include <libexplain/setsockopt.h>

#include <lib/mail_transfer_agent.h>


void
mail_transfer_agent::daemon_glue(const string_list_t &names)
{
    if (!names.empty())
        no_names_on_command_line();
    daemon();
}


void
mail_transfer_agent::daemon_nofork_glue(const string_list_t &names)
{
    set("debug", true);
    daemon_glue(names);
}


void
mail_transfer_agent::daemon(int argc, char **argv)
{
    explain_program_name_set(argv[0]);
    for (;;)
    {
        int c = getopt(argc, argv, "C:O:o:");
        if (c == EOF)
            break;
        switch (c)
        {
        case 'C':
            set_configuration_file(optarg);
            break;

        case 'O':
        case 'o':
            parse_and_set(optarg);
            break;

        default:
            usage_without_names();
            // NOTREACHED
        }
    }
    if (optind != argc)
        usage_without_names();
    daemon();
}


void
mail_transfer_agent::daemon()
{
    //
    // Determine which port is to be used.
    //
    int port_number = 25;
    std::string port_name = get("port");
    if (!port_name.empty())
    {
        struct servent *sep = getservbyname(port_name.c_str(), 0);
        if (sep)
        {
            port_number = sep->s_port;
        }
        else
        {
            port_number = strtol(port_name.c_str(), 0, 0);
            if (port_number < 1 || port_number >= 65536)
            {
                explain_output_error_and_die
                (
                    "port \"%s\" invalid",
                    port_name.c_str()
                );
            }
        }
    }

    //
    // Create the socket
    //
    int fd = explain_socket_or_die(PF_INET, SOCK_STREAM, IPPROTO_TCP);

    //
    // Don't give an error when re-using a recently used address.
    //
    int one = 1;
    explain_setsockopt_or_die(fd, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));

    //
    // Bind to the port we want.
    //
    struct sockaddr_in addr;
    memset(&addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = htonl(INADDR_ANY);
    addr.sin_port = htons(port_number);
    explain_bind_or_die(fd, (struct sockaddr *)&addr, sizeof(addr));

    //
    // Tell the system we are listening for connections
    //
    explain_listen_or_die(fd, 5);

    //
    // Most of the time, you will want
    // to fork and run the daemon in the background.
    //
    if (!get_bool("foreground"))
    {
        int pid = explain_fork_or_die();
        if (pid != 0)
            exit(0);

        //
        // redirect errors to syslog
        //
        int option = LOG_CONS | LOG_PID;
        int facility = LOG_MAIL;
        int level = LOG_ERR;
        explain_output_register
        (
            explain_output_syslog_new3(option, facility, level)
        );
    }

    //
    // don't collect the exit status of finished child processes.
    //
    signal(SIGCHLD, SIG_IGN);

    //
    // write the process ID to a file so that it is easy to find and kill.
    //

    //
    // now just wait for connections to come in
    //
    for (;;)
    {
        struct sockaddr addr;
        memset(&addr, 0, sizeof(addr));
        socklen_t addrlen = sizeof(addr);
        int cfd = accept(fd, &addr, &addrlen);
        if (cfd < 0)
        {
            int err = errno;
            if (err == EAGAIN)
                return;
            explain_output_error_and_die
            (
                "%s",
                explain_errno_accept(errno, fd, &addr, &addrlen)
            );
        }

        //
        // create new process for the server
        //
        int pid = explain_fork_or_die();
        if (pid == 0)
        {
            //
            // redirect errors to syslog
            // if they aren't already
            //
            int option = LOG_CONS | LOG_PID;
            int facility = LOG_MAIL;
            int level = LOG_ERR;
            explain_output_register
            (
                explain_output_syslog_new3(option, facility, level)
            );

            //
            // redirect stdio from socket
            //
            close(0);
            explain_dup2_or_die(cfd, 0);
            close(1);
            explain_dup2_or_die(cfd, 1);

            //
            // Be a server.
            //
            server();
            exit(0);
        }
        close(cfd);
    }
}


// vim: set ts=8 sw=4 et :
