//
//      black-hole-mta - email goes in but nothing comes out
//      Copyright (C) 2007, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License as
//      published by the Free Software Foundation; either version 3 of
//      the License, or (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <boost/algorithm/string.hpp>
#include <cstdio>

#include <lib/hostname.h>
#include <lib/mail_transfer_agent.h>


void
mail_transfer_agent::server_glue(const string_list_t &names)
{
    if (!names.empty())
        no_names_on_command_line();
    server();
}


static std::string
first_word(const char *s)
{
    while (isspace((unsigned char)*s))
        ++s;
    const char *end = s;
    while (*end && !isspace((unsigned char)*end))
        ++end;
    return std::string(s, end - s);
}


void
mail_transfer_agent::server()
{
    // ignore any -i on the command line
    set("no-dot", false);

    // See RFC821 "Simple Mail Transfer Protocol"
    printf("220 %s\r\n", hostname());
    fflush(stdout);
    string_list_t names;
    for (;;)
    {
        char line[1000];
        if (!fgets(line, sizeof(line), stdin))
            break;
        size_t line_len = strlen(line);
        if (line_len > 0 && isspace((unsigned char)line[line_len - 1]))
            line[--line_len] = 0;
        std::string cmd = first_word(line);
        boost::to_upper(cmd);
        if (cmd == "HELO")
        {
            // S: 250
            // E: 500, 501, 504, 421
            printf("250 %s\r\n", hostname());
        }
        else if (cmd == "MAIL")
        {
            // S: 250
            // F: 552, 451, 452
            // E: 500, 501, 421
            //
            // we expect a line which starts "MAIL FROM:"
            // but we are going to accept anything at the moment
            printf("250 OK\r\n");
        }
        else if (cmd == "RCPT")
        {
            // S: 250, 251
            // F: 550, 551, 552, 553, 450, 451, 452
            // E: 500, 501, 503, 421
            char *s = line + 4;
            while (isspace((unsigned char)*s))
                ++s;
            if (0 == memcmp(s, "TO:", 3))
                s += 3;
            names.push_back(std::string(s));
            printf("250 OK\r\n");
        }
        else if (cmd == "DATA")
        {
            // I: 354 -> data -> S: 250
            // F: 552, 554, 451, 452
            // F: 451, 554
            // E: 500, 501, 503, 421
            if (names.empty())
            {
                printf("503 No recipients specified\r\n");
            }
            else
            {
                printf("354 Send data, end with <CRLF>.<CRLF>\r\n");
                fflush(stdout);
                send_glue(names);
                printf("250 OK\r\n");
            }
        }
        else if (cmd == "RSET")
        {
            // S: 250
            // E: 500, 501, 504, 421
            names.clear();
            printf("250 OK\r\n");
        }
        else if (cmd == "SEND")
        {
            // optional
            // S: 250
            // F: 552, 451, 452
            // E: 500, 501, 502, 421
            printf("502 Command not implemented\r\n");
        }
        else if (cmd == "SOML")
        {
            // optional
            // S: 250
            // F: 552, 451, 452
            // E: 500, 501, 502, 421
            printf("502 Command not implemented\r\n");
        }
        else if (cmd == "SAML")
        {
            // optional
            // S: 250
            // F: 552, 451, 452
            // E: 500, 501, 502, 421
            printf("502 Command not implemented\r\n");
        }
        else if (cmd == "VRFY")
        {
            // optional
            // S: 250, 251
            // F: 550, 551, 553
            // E: 500, 501, 502, 504, 421
            printf("502 Command not implemented\r\n");
        }
        else if (cmd == "EXPN")
        {
            // optional
            // S: 250
            // F: 550
            // E: 500, 501, 502, 504, 421
            printf("502 Command not implemented\r\n");
        }
        else if (cmd == "HELP")
        {
            // optional
            // S: 211, 214
            // E: 500, 501, 502, 504, 421
            printf("502 Command not implemented\r\n");
        }
        else if (cmd == "NOOP")
        {
            // S: 250
            // E: 500, 421
            printf("250 OK\r\n");
        }
        else if (cmd == "QUIT")
        {
            // S: 221
            // E: 500
            printf("221 Bye\r\n");
            fflush(stdout);
            break;
        }
        else if (cmd == "TURN")
        {
            // optional
            // S: 250
            // F: 502
            // E: 500, 503
            printf("502 Command not implemented\r\n");
        }
        else
        {
            printf("502 Command \"%s\" unknown\r\n", cmd.c_str());
        }
        fflush(stdout);
        if (ferror(stdout))
            break;
    }
}
