//
// black-hole-mta - email goes in but nothing comes out
// Copyright (C) 2007, 2009, 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/config.h>
#include <cerrno>
#include <cstring>
#include <libexplain/fopen.h>
#include <libexplain/output.h>

#include <lib/libdir.h>
#include <lib/mail_transfer_agent.h>
#include <lib/rfc822.h>
#include <lib/sizeof.h>


void
mail_transfer_agent::parse_and_set(const char *arg)
{
    //
    // Maybe we should whinge if we are given the empty string, but for
    // now we will just ignore it.
    //
    if (!*arg)
        return;

    //
    // As a special legacy case, a single letter option
    // means set it to 1 (true).
    //
    if (arg[1] == '\0')
    {
        set(arg, "1");
        return;
    }

    const char *eq = strchr(arg, '=');
    if (!eq)
    {
        //
        // If there is no equals sign, the the name is the first
        // character, and the value is the rest of the string.  Yes,
        // it's stupid, it's historical.
        //
        std::string name(arg, 1);
        std::string value(arg + 1);
        set(name, value);
        return;
    }

    std::string name(arg, eq - arg);
    std::string value(eq + 1);
    set(name, value);
}


void
mail_transfer_agent::set(const std::string &name, const std::string &value)
{
    db[name] = value;
}


void
mail_transfer_agent::set(const std::string &name, bool value)
{
    db[name] = (value ? "true" : "false");
}


void
mail_transfer_agent::set_unless(const std::string &name,
    const std::string &value)
{
    if (db.find(name) == db.end())
        db[name] = value;
}


std::string
mail_transfer_agent::get(const std::string &name, const std::string &dflt)
{
    read_configuration_file();
    if (db.find(name) == db.end())
    {
        // remember the value for next time
        db[name] = dflt;
        return dflt;
    }
    return db[name];
}


static bool
to_bool(const std::string &s)
{
    size_t len = s.size();
    const char *cp = s.c_str();

    struct table_t
    {
        const char *name;
        bool value;
    };

    static const table_t table[] =
    {
        { "true", true },
        { "yes", true },
        { "false", false },
        { "no", false },
    };

    for (const table_t *tp = table; tp < ENDOF(table); ++tp)
    {
        size_t len2 = strlen(tp->name);
        if (len2 <= len && 0 == memcmp(tp->name, cp, len2))
            return tp->value;
    }

    long n = strtol(cp, 0, 0);
    return (n != 0);
}


bool
mail_transfer_agent::get_bool(const std::string &name, bool dflt)
{
    read_configuration_file();
    if (db.find(name) == db.end())
    {
        // remember the value for next time
        set(name, dflt);
        return dflt;
    }
    return to_bool(db[name]);
}


void
mail_transfer_agent::read_configuration_file()
{
    if (have_read_configuration_file)
        return;
    have_read_configuration_file = true;

    if (configuration_file.empty())
    {
        std::string cfg("/black-hole-mta.conf");
        configuration_file = configured_sysconfdir() + cfg;
    }

    //
    // Read the configuration file.
    //
    rfc822 cfg;
    if (!cfg.load(configuration_file))
    {
        int err = errno;
        if (err != ENOENT)
        {
            explain_output_error_and_die
            (
                "%s",
                explain_errno_fopen(err, configuration_file.c_str(), "r")
            );
        }
    }

    //
    // Transfer the contents, being careful not to contradict the
    // command line.
    //
    const db_t &r = cfg.get();
    for (db_t::const_iterator it = r.begin(); it != r.end(); ++it)
    {
        set_unless(it->first, it->second);
    }
}


// vim: set ts=8 sw=4 et :
