//
//      black-hole-mta - email goes in but nothing comes out
//      Copyright (C) 2007, 2009 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License as
//      published by the Free Software Foundation; either version 3 of
//      the License, or (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cerrno>
#include <cstdio>
#include <boost/algorithm/string.hpp>

#include <lib/rfc822.h>


rfc822::~rfc822()
{
}


rfc822::rfc822()
{
}


rfc822::rfc822(const rfc822 &arg) :
    data(arg.data)
{
}


rfc822 &
rfc822::operator=(const rfc822 &arg)
{
    if (this != &arg)
        data = arg.data;
    return *this;
}


std::string
rfc822::get(const std::string &key, const std::string &dflt)
    const
{
    std::string ukey = boost::to_lower_copy(key);
    data_t::const_iterator it = data.find(ukey);
    if (it == data.end())
        return dflt;
    return (*it).second;
}


void
rfc822::set(const std::string &key, const std::string &value)
{
    data[boost::to_lower_copy(key)] = value;
}


bool
rfc822::load(const std::string &filename)
{
    FILE *fp = fopen(filename.c_str(), "r");
    if (!fp)
        return false;
    int num_items = 0;
    for (;;)
    {
        std::string key;
        int ic = getc(fp);
        if (ic == EOF)
        {
            int err = errno;
            fclose(fp);
            errno = err;
            return (num_items != 0);
        }
        unsigned char c = ic;
        if (c == '\n')
        {
            fclose(fp);
            return true;
        }

        // Get the name.
        // It must be composed of [-_A-Za0z0-9]
        std::string name;
        for (;;)
        {
            switch (c)
            {
            case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
            case 'G': case 'H': case 'I': case 'J': case 'K': case 'L':
            case 'M': case 'N': case 'O': case 'P': case 'Q': case 'R':
            case 'S': case 'T': case 'U': case 'V': case 'W': case 'X':
            case 'Y': case 'Z':
            case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
            case 'g': case 'h': case 'i': case 'j': case 'k': case 'l':
            case 'm': case 'n': case 'o': case 'p': case 'q': case 'r':
            case 's': case 't': case 'u': case 'v': case 'w': case 'x':
            case 'y': case 'z':
            case '0': case '1': case '2': case '3': case '4':
            case '5': case '6': case '7': case '8': case '9':
            case '-': case '_':
                name += c;

                ic = getc(fp);
                if (ic == EOF)
                {
                    invalid_input:
                    fclose(fp);
                    errno = EINVAL;
                    return false;
                }
                c = ic;
                continue;

            default:
                break;
            }
            break;
        }

        //
        // Next we should see a colon followed by optional white space.
        //
        if (c != ':')
            goto invalid_input;
        ic = getc(fp);
        if (ic == EOF)
        {
            // have name and colon, but no value
            name_no_value:
            set(name, "");
            fclose(fp);
            return true;
        }
        c = ic;

        while (c == '\t' || c == ' ')
        {
            ic = getc(fp);
            if (ic == EOF)
                goto name_no_value;
            c = ic;
        }

        //
        // and now we are looking for the value
        // (be aware of continuation lines)
        //
        std::string value;
        for (;;)
        {
            if (c == '\n')
            {
                ic = getc(fp);
                if (ic == EOF)
                    break;
                ungetc(ic, fp);
                if (ic != ' ' && ic != '\t')
                    break;
            }
            value += c;

            ic = getc(fp);
            if (ic == EOF)
                break;
            c = ic;
        }

        //
        // remember the name and value pair
        //
        set(name, value);
    }
}


void
rfc822::clear()
{
    data.clear();
}
