//
//      black-hole-mta - email goes in but nothing comes out
//      Copyright (C) 2007, 2011 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License as
//      published by the Free Software Foundation; either version 3 of
//      the License, or (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef LIB_RFC822_H
#define LIB_RFC822_H

#include <string>
#include <map>

/**
  * The rfc822 class is used to represent the content of RFC822 headers
  * read from a file.
  */
class rfc822
{
public:
    /**
      * The destructor.
      */
    virtual ~rfc822();

    /**
      * The default constructor.
      */
    rfc822();

    /**
      * The copy constructor.
      */
    rfc822(const rfc822 &);

    /**
      * The assignment operator.
      */
    rfc822 &operator=(const rfc822 &);

    /**
      * The get method may be used to obtain a value.
      *
      * @param key
      *     The name given
      * @returns
      *     the string of the value, or the empty string if this name
      *     is not known.
      */
    std::string get(const std::string &key, const std::string &dflt = "") const;

    /**
      * The set method may be used to remember a name/value pair.
      *
      * @param key
      *     The name to be assigned
      * @param value
      *     The value to be assigned
      */
    void set(const std::string &key, const std::string &value);

    /**
      * The clear method may be used to dicard all name/value pairs.
      */
    void clear();

    /**
      * The load method may be used to load the RFC822 headers from the
      * given file.
      *
      * @param filename
      *     The name of the file to be read.
      * @returns
      *     true on success, false (plus errno) on failure
      */
    bool load(const std::string &filename);

    typedef std::map<std::string, std::string> data_t;

    /**
      * The get method is used to access the complete set of names and
      * their values.
      */
    const data_t &get() const { return data; }

private:
    /**
      * The data instance variable is used to remember the name/value pairs.
      */
    data_t data;
};

// vim: set ts=8 sw=4 et :
#endif // LIB_RFC822_H
